(* $Id: omega_QCD.ml 1340 2009-12-02 23:45:04Z ohl $

   Copyright (C) 1999-2009 by

       Wolfgang Kilian <kilian@hep.physik.uni-siegen.de>
       Thorsten Ohl <ohl@physik.uni-wuerzburg.de>
       Juergen Reuter <juergen.reuter@physik.uni-freiburg.de>

   WHIZARD is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   WHIZARD is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  *)

let rcs_file = RCS.parse "F90_QCD" ["QCD"]
    { RCS.revision = "$Revision: 1340 $";
      RCS.date = "$Date: 2009-12-03 00:45:04 +0100 (Thu, 03 Dec 2009) $";
      RCS.author = "$Author: ohl $";
      RCS.source
        = "$Source: /home/sources/ohl/ml/omega/src/omega_QCD.ml,v $" }

(* QCD with colors. *)

module M : Model.T =
  struct
    let rcs = rcs_file

    open Coupling

    let options = Options.empty
        
    type flavor =
      | U | Ubar | D | Dbar
      | C | Cbar | S | Sbar
      | T | Tbar | B | Bbar
      | Gl 
         
    type flavor_sans_color = flavor
    let flavor_sans_color f = f

    let external_flavors () = 
      [ "Quarks", [U; D; C; S; T; B; Ubar; Dbar; Cbar; Sbar; Tbar; Bbar]; 
        "Gauge Bosons", [Gl]]
    let flavors () = ThoList.flatmap snd (external_flavors ())

    type gauge = unit
    type constant = Gs | G2 | I_Gs

    let lorentz = function
      | U | D | C | S | T | B -> Spinor
      | Ubar | Dbar | Cbar | Sbar | Tbar | Bbar -> ConjSpinor 
      | Gl -> Vector

    let color = function
      | U | D | C | S | T | B -> Color.SUN 3
      | Ubar | Dbar | Cbar | Sbar | Tbar | Bbar -> Color.SUN (-3)
      | Gl -> Color.AdjSUN 3

    let propagator = function
      | U | D | C | S | T | B -> Prop_Spinor
      | Ubar | Dbar | Cbar | Sbar | Tbar | Bbar -> Prop_ConjSpinor 
      | Gl -> Prop_Feynman

    let width _ = Timelike

    let goldstone _ =
      None

    let conjugate = function
      | U -> Ubar
      | D -> Dbar
      | C -> Cbar
      | S -> Sbar
      | T -> Tbar
      | B -> Bbar
      | Ubar -> U 
      | Dbar -> D 
      | Cbar -> C 
      | Sbar -> S 
      | Tbar -> T 
      | Bbar -> B 
      | Gl -> Gl 

    let conjugate_sans_color = conjugate

    let fermion = function
      | U | D | C | S | T | B -> 1
      | Ubar | Dbar | Cbar | Sbar | Tbar | Bbar -> -1
      | Gl -> 0

    module F = Modeltools.Fusions (struct
      type f = flavor
      type c = constant
      let compare = compare
      let conjugate = conjugate
    end)

(* This is compatible with CD+. *)

    let color_current =
      [ ((Dbar, Gl, D), FBF ((-1), Psibar, V, Psi), Gs);
        ((Ubar, Gl, U), FBF ((-1), Psibar, V, Psi), Gs);
        ((Cbar, Gl, C), FBF ((-1), Psibar, V, Psi), Gs);
        ((Sbar, Gl, S), FBF ((-1), Psibar, V, Psi), Gs);
        ((Tbar, Gl, T), FBF ((-1), Psibar, V, Psi), Gs);
        ((Bbar, Gl, B), FBF ((-1), Psibar, V, Psi), Gs)]

   let three_gluon =
      [ ((Gl, Gl, Gl), Gauge_Gauge_Gauge 1, Gs)]

    let gauge4 = Vector4 [(2, C_13_42); (-1, C_12_34); (-1, C_14_23)]

    let four_gluon = 
      [ ((Gl, Gl, Gl, Gl), gauge4, G2)]

    let vertices3 = 
      (color_current @ three_gluon)
       
   let vertices4 = four_gluon

    let vertices () =
      (vertices3, vertices4, [])
      
    let table = F.of_vertices (vertices ())
    let fuse2 = F.fuse2 table
    let fuse3 = F.fuse3 table
    let fuse = F.fuse table
    let max_degree () = 4

    let parameters () = { input = [Gs, 1.0]; derived = []; derived_arrays = [] } 
    let flavor_of_string = function
      | "u" -> U 
      | "d" -> D 
      | "c" -> C 
      | "s" -> S 
      | "t" -> T 
      | "b" -> B 
      | "ubar" -> Ubar
      | "dbar" -> Dbar
      | "cbar" -> Cbar
      | "sbar" -> Sbar
      | "tbar" -> Tbar
      | "bbar" -> Bbar
      | "gl" -> Gl 
      | _ -> invalid_arg "Models.QCD.flavor_of_string"

    let flavor_to_string = function
      | U -> "u" 
      | Ubar -> "ubar" 
      | D -> "d" 
      | Dbar -> "dbar" 
      | C -> "c" 
      | Cbar -> "cbar" 
      | S -> "s" 
      | Sbar -> "sbar" 
      | T -> "t" 
      | Tbar -> "tbar" 
      | B -> "b" 
      | Bbar -> "bbar" 
      | Gl -> "gl" 

    let flavor_to_TeX = function
      | U -> "u" 
      | Ubar -> "\bar{u}" 
      | D -> "d" 
      | Dbar -> "\bar{d}" 
      | C -> "c" 
      | Cbar -> "bar{c}" 
      | S -> "s" 
      | Sbar -> "\bar{s}" 
      | T -> "t" 
      | Tbar -> "\bar{t}" 
      | B -> "b" 
      | Bbar -> "\bar{b}" 
      | Gl -> "g" 

    let flavor_symbol = function
      | U -> "u"
      | Ubar -> "ubar"
      | D -> "d"
      | Dbar -> "dbar"
      | C -> "c"
      | Cbar -> "cbar"
      | S -> "s"
      | Sbar -> "sbar"
      | T -> "t"
      | Tbar -> "tbar"
      | B -> "b"
      | Bbar -> "bbar"
      | Gl -> "gl" 

    let flavor_sans_color_of_string = flavor_of_string
    let flavor_sans_color_to_string = flavor_to_string
    let flavor_sans_color_to_TeX = flavor_to_TeX
    let flavor_sans_color_symbol = flavor_symbol

    let gauge_symbol () =
      failwith "Models.QCD.gauge_symbol: internal error"

    let pdg = function
      | D -> 1 | Dbar -> -1
      | U -> 2 | Ubar -> -2
      | S -> 3 | Sbar -> -3
      | C -> 4 | Cbar -> -4
      | B -> 5 | Bbar -> -5
      | T -> 6 | Tbar -> -6
      | Gl -> 21 

    let mass_symbol f = 
      "mass(" ^ string_of_int (abs (pdg f)) ^ ")"

    let width_symbol f =
      "width(" ^ string_of_int (abs (pdg f)) ^ ")"

    let constant_symbol = function
      | I_Gs -> "(0,1)*gs"
      | Gs -> "gs"
      | G2 -> "gs**2"
  end

module O = Omega.Make(Fusion.Mixed23)(Targets.Fortran)(M)

let _ = O.main ()


(*i
 *  Local Variables:
 *  mode:caml
 *  indent-tabs-mode:nil
 *  page-delimiter:"^(\\* .*\n"
 *  End:
i*)
